/* AMBGRO - program for convering AMBER topology to GROMACS topology */
/* Filip Ryjacek 2002 */
/* BDR Modified according to GMX mailing list article by Tomas Kuber
   to allow proper size of file name string allocations */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>

#include "ambconv.h"


/* open file in mode 'mode' */
FILE* open_file(char* filename, char* mode)
  {
    FILE* file;
    file = fopen(filename, mode);
/*      if (file == NULL || ferror(file)!=0)
        {
          printerror(filename);	
          printerror("Cannot open file");
          exit(1);
        } */
    return(file);
  }

/* close file */
void close_file(FILE* file)
  {
    if (fclose(file) == EOF) printerror("Cannot close file");
  }

/* print error string */
void printerror(char* message)
  {
    extern int errno;
    printf("Error: %s: %s\n", message, strerror(errno));
  }

/* read command line */
/* tohle chce predelat ve stylu parse_comand_args z GROMACSU */
void read_commandline(int argc, char* argv[])
  {
    int i;
    /* initialize */
    sol = 1;
    nbpairs = 1;
    constraints = 1;
    lacc_rst = 1;
    position_constraints = 1;
    gromacs = 1;
    tinker = 1;
    pdb = 1;
    invert = 1;
    
      if (argc < 11)
        {
	  printf("Too few command line arguments!");
//	  exit(1);
	  /* abort; */
	}
    
      for (i=1; i<argc; i++)
        {

          if (strcmp(argv[i], "-v") == 0) 
	    {
		printf("AMBCONV v%1.1f\n", version);
	    }

          if (strcmp(argv[i], "-at") == 0) 
	    {
	      ambtop_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      ambtop_filename = strcpy(ambtop_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "AMBER .top", ambtop_filename);
#endif
	    }
	    
          if (strcmp(argv[i], "-gt") == 0) 
	    {
	      grotop_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      grotop_filename = strcpy(grotop_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "GROMACS .top", grotop_filename);
#endif
              gromacs = 0;
	    }
	    
          if (strcmp(argv[i], "-tt") == 0) 
	    {
	      tintop_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      tintop_filename = strcpy(tintop_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "TINKER .top", tintop_filename);
#endif
              tinker = 0;
	    }

          if (strcmp(argv[i], "-rst") == 0) 
	    {
	      rst_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      rst_filename = strcpy(rst_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "AMBER .rst", rst_filename);
#endif
	    }
	    
          if (strcmp(argv[i], "-gro") == 0) 
	    {
	      gro_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      gro_filename = strcpy(gro_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "GROMACS .gro", gro_filename);
#endif
              gromacs = 0;
	    }

          if (strcmp(argv[i], "-xyz") == 0) 
	    {
	      xyz_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      xyz_filename = strcpy(xyz_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "TINKER .xyz", xyz_filename);
#endif
              tinker = 0;
	    }

          if (strcmp(argv[i], "-pdb") == 0) 
	    {
	      pdb_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      pdb_filename = strcpy(pdb_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "PDB .pdb", pdb_filename);
#endif
              pdb = 0;
	    }
	    
          if (strcmp(argv[i], "-common") == 0) 
	    {
	      common_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      common_filename = strcpy(common_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "COMMON", common_filename);
#endif
	    }
	    
          if (strcmp(argv[i], "-sol") == 0) 
	    {
	      sol_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      sol_filename = strcpy(sol_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "SOLVENT", sol_filename);
#endif
	      sol = 0;
	    }
	    
          if (strcmp(argv[i], "-nbpairs") == 0) 
	    {
              nbpairs = 0;
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "NBPAIRS","true");
#endif
	    }
	    
          if (strcmp(argv[i], "-const") == 0) 
	    {
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "CONSTRAINTS","true");
#endif
              constraints = 0;
	    }

          if (strcmp(argv[i], "-invert") == 0) 
	    {
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "INVERT","true");
#endif
              invert = 0;
	    }

          if (strcmp(argv[i], "-pconst") == 0) 
	    {
	      constraints_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      constraints_filename = strcpy(constraints_filename, argv[i+1]);
              position_constraints = 0;
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "POSITION CONSTRAINTS", constraints_filename);
#endif
              gromacs = 0;
	        constraints_file = open_file(constraints_filename, "w");
		fprintf(constraints_file,"[ position_restraints ]\n");
		fprintf(constraints_file,"; atom    func     fc_x    fc_y    fc_z\n");
		close_file(constraints_file);
	    }

          if (strcmp(argv[i], "-lacc_rst") == 0) 
	    {
	      lacc_rst_filename = (char*) malloc(strlen(argv[i+1])*sizeof(char)+1);
	      lacc_rst_filename = strcpy(lacc_rst_filename, argv[i+1]);
#ifdef DEBUG_UTIL
	      printf("%-25s: %-32s\n", "LOW ACCURACY .rst", lacc_rst_filename);
#endif
              lacc_rst = 0;
	      gromacs = 0;
	    }

	} 
  }

/* read string from top */
void readstring(unsigned int length, char* source)
  {
    int ch;
    unsigned int counter=0;

    while (counter < length)
      {
        ch = fgetc(ambtop);
        if (ch == '\n') continue;
	if (ch == EOF) { printerror("EOF while reading from AMBER TOP file"); exit(1); }
	source[counter] = (char) ch;
	counter++;	
      }

    source[counter] = '\0'; /* +1   */
  }


/* replace malloc with xmalloc */
void* xmalloc(size_t size)
  {
    void* value = malloc(size);
    if (value == 0) printerror("Memory allocation error");
#ifdef DEBUG_MEM
    printf("\nxmalloc:%p (%dbytes)\n", value, size);
#endif
    return(value);
  }

/* replace free with xfree */
void xfree(void* mem)
  {
    free(mem);
    if (errno != 0) printerror("Memory release error!");
#ifdef DEBUG_MEM
    printf("\nxfree:%p\n", mem);
#endif
  }
/*
// print part of array input
char* get_slice(char* input, unsigned int length, unsigned int distance, unsigned int index)
  {
    unsigned int start;
    char target_buffer[5];
    char* target;
    
    // catch possible errors
    if (strlen(input) < length) printerror("get_slice: input < length");
  
    start = index*distance;
    target = target_buffer;
    target = memset(target,'\0',length);

    target = strncpy(target, input+(index*distance), length);
    target[length] = '\0';

#ifdef DEBUG_UTIL
//    printf("get_slice |%s| from |%s| (%d,%d)\n", target, input, start, start+length-1);
#endif
    return target;
  }
*/

// print part of array input
char* get_slice(char* input, char* output, unsigned int length, unsigned int distance, unsigned int index)
  {
    unsigned int start;
    
    // catch possible errors
    if (strlen(input) < length) printerror("get_slice: input < length");
  
    start = index*distance;
    output = strncpy(output, input+(index*distance), length);
    output[length] = '\0';

#ifdef DEBUG_SLICE
//    printf("get_slice |%s| from |%s| (%d,%d)\n", target, input, start, start+length-1);
#endif
    return output;
  }



/* returns 1 if atom is between 'from' and 'to'  */
unsigned int out(int atom, int from, int to)
  {
    if ((atom < from) || (atom > to))
      {
        return 1;
      }
    else
      {
        return 0;
      }
  }


unsigned int not_ekviv(double a, double b, double limit)
  {
    if(fabs(a-b) > limit)
      {
        return 1;
      }
      
    else
     {
       return 0;
     }
  }

unsigned int i_not_ekviv(int a, int b)
  {
    if(a != b)
      {
        return 1;
      }
      
    else
     {
       return 0;
     }
  }

/**
* Replace apostrophe in input_string with "q".
  Function searches 'input_string' by 'strrchr' function
  and replace each occurence of the apostrophe
*/
char* repapo(char* input_string)
{
  char *apostrophe;
  apostrophe = strrchr(input_string, '\'');
    if (apostrophe != '\0')
    {
      *apostrophe = 'q';
    }
return input_string;
}

