/* AMBGRO - program for convering AMBER RST coordinates to GROMACS GRO coordinates */
/* Filip Ryjacek 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
/* #include <mcheck.h> */

#include "ambconv.h"


/* write GRO file  */
void write_gro(s_atop* atop)
  {
    FILE* gro;
    unsigned int counter;
    /* atom type */
    char* atomname;
    char* atomtype;
    unsigned int atomindex, natom;
    /* residue parameters */
    unsigned int resindex, nres, from, to;
    char* resname;
    /* coords + veloc */
    double x,y,z;
    double vx, vy, vz;
    
    /* allocate space labels */    
    resname = (char*) xmalloc(5);
    atomname = (char*) xmalloc(5);
    atomtype = (char*) xmalloc(5);


    /* debug memory allocation */
#ifdef DEBUG_MEM
  mtrace();
#endif
#ifdef DEBUG_GRO
  printf("\nopening %s...\n", gro_filename);
#endif

    /* open file */
    gro = open_file(gro_filename, "w");

/* write title + number of atoms */
    if (atop->notitle == 0)
      {
        fprintf(gro, "%s\n", rst_title);
      }
    else
      {
        fprintf(gro, "no title\n");
      }

    fprintf(gro, "%5d\n", atop->natom);

    nres = atop->nres;
    /* go through all residues */
    for(resindex=0; resindex< nres; resindex++)
      {
        /* get residue label and bounds */
        resname = get_slice(atop->labres,resname,4,4,resindex);

        from = atop->ipres[resindex]-1;
          if (resindex == atop->nres-1)
          {
            to = atop->natom-1;
          }
        else
          {
            to = (atop->ipres[resindex+1]-1)-1;
          }

        natom = to-from;
        /* go through all atoms */
	counter = 1;
        for ( atomindex=from; atomindex <= to; atomindex++)
          {
            /* get atom name&type */
            atomname = get_slice(atop->igraph, atomname, 4, 4, atomindex);
            atomname = repapo(atomname);
	    	    
            atomtype = get_slice(atop->isymbl, atomtype, 4, 4, atomindex);

            x = coords[atomindex*3];
	    y = coords[(atomindex*3)+1];
	    z = coords[(atomindex*3)+2];

	    x = x*100.0000;
	    y = y*100.0000;
	    z = z*100.0000;

	    x = rint(x)*0.001;
	    y = rint(y)*0.001;
	    z = rint(z)*0.001;

            vx = vy = vz = 0.0000;
	   
            fprintf(gro,"%5d%5s%5s%5d%8.3f%8.3f%8.3f%8.4f%8.4f%8.4f\n", resindex+1,resname,atomname,counter,x,y,z,vx,vy,vz);
	    
	    counter++;
          }
      }

// close the file
close_file(gro);

//#ifdef USE_FREE
free(atomname);
free(atomtype);
free(resname);
//#endif

#ifdef DEBUG_GRO
  printf("%s closed.\n", gro_filename);
#endif
#ifdef DEBUG_MEM
  muntrace();
#endif
}

