/* AMBGRO - program for convering AMBER RST coordinates to GROMACS GRO coordinates */
/* Filip Ryjacek 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
/* #include <mcheck.h> */

#include "ambconv.h"


/* write low accuracy .RST file  */
void write_lacc_rst(s_atop* atop)
  {
    FILE* lacc_rst;
    unsigned int counter;
    /* atom type */
    char* atomname;
    char* atomtype;
    unsigned int atomindex, natom;
    /* residue parameters */
    unsigned int resindex, nres, from, to;
    char* resname;
    /* coords + veloc */
    double x,y,z;
    /* misc */
    unsigned int flag;
    
    /* allocate space labels */    
    resname = (char*) xmalloc(5);
    atomname = (char*) xmalloc(5);
    atomtype = (char*) xmalloc(5);

    flag = 1;

    /* debug memory allocation */
#ifdef DEBUG_MEM
  mtrace();
#endif
#ifdef DEBUG_GRO
  printf("\nopening %s...\n", lacc_rst_filename);
#endif

    /* open file */
    lacc_rst = open_file(lacc_rst_filename, "w");

/* write title + number of atoms */
    if (atop->notitle == 0)
      {
        fprintf(lacc_rst, "%s\n", rst_title);
      }
    else
      {
        fprintf(lacc_rst, "no title\n");
      }

    fprintf(lacc_rst, "%5d\n", atop->natom);

    nres = atop->nres;
    /* go through all residues */
    for(resindex=0; resindex< nres; resindex++)
      {
        /* get residue label and bounds */
        resname = get_slice(atop->labres, resname, 4,4,resindex);

        from = atop->ipres[resindex]-1;
          if (resindex == atop->nres-1)
          {
            to = atop->natom-1;
          }
        else
          {
            to = (atop->ipres[resindex+1]-1)-1;
          }

        natom = to-from;
        /* go through all atoms */
	counter = 1;
        for ( atomindex=from; atomindex <= to; atomindex++)
          {
            /* get atom name&type */
            atomname = get_slice(atop->igraph, atomname, 4, 4, atomindex);
            atomtype = get_slice(atop->isymbl, atomtype, 4, 4, atomindex);

            x = coords[atomindex*3];
	    y = coords[(atomindex*3)+1];
	    z = coords[(atomindex*3)+2];

	    x = x*100.0000;
	    y = y*100.0000;
	    z = z*100.0000;

	    x = rint(x)*0.01;	    
	    y = rint(y)*0.01;	    
	    z = rint(z)*0.01;	    

            fprintf(lacc_rst,"%12.7f%12.7f%12.7f", x,y,z);
            if (flag%2 == 0) {
              fprintf(lacc_rst,"\n");
	    }
	    
	    counter++;
	    flag++;
          }
      }

/* close file and free memory */
close_file(lacc_rst);

#ifdef USE_FREE
free(atomname);
free(atomtype);
free(resname);
#endif

#ifdef DEBUG_GRO
  printf("all O.K., %s closed\n", lacc_rst_filename);
#endif
#ifdef DEBUG_MEM
  muntrace();
#endif
}

