/* AMBGRO - program for convering AMBER RST coordinates to GROMACS GRO coordinates */
/* Filip Ryjacek 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
/* #include <mcheck.h> */

#include "ambconv.h"

/* write PDB file  */
void write_pdb(s_atop* atop)
  {
    FILE* pdb;
    unsigned int counter;
    /* atom type */
    char* atomname;
    unsigned int atomindex, natom;
    /* residue parameters */
    unsigned int resindex, nres, from, to;
    char* resname;
    /* coords */
    double x,y,z;
    
    /* allocate space for labels */    
    resname = (char*) xmalloc(5);
    atomname = (char*) xmalloc(5);

    /* debug memory allocation */
#ifdef DEBUG_MEM
  mtrace();
#endif
#ifdef DEBUG_GRO
  printf("\nopening %s...\n", pdb_filename);
#endif

    /* open file */
    pdb = open_file(pdb_filename, "w");

/* write title + number of atoms */
    if (atop->notitle == 0)
      {
        fprintf(pdb, "REM %s\n", rst_title);
      }
    else
      {
        fprintf(pdb, "REM no title\n");
      }
    fprintf(pdb, "REM number of atoms:%5d\n", atop->natom);
    fprintf(pdb, "REM number of residues:%5d\n", atop->nres);
    fprintf(pdb, "REM\n");

    nres = atop->nres;
    /* go through all residues */
    for(resindex=0; resindex< nres; resindex++)
      {
        /* get residue label and bounds */
        resname = get_slice(atop->labres, resname, 4,4,resindex);

        from = atop->ipres[resindex]-1;
          if (resindex == atop->nres-1)
          {
            to = atop->natom-1;
          }
        else
          {
            to = (atop->ipres[resindex+1]-1)-1;
          }

        natom = to-from;
        /* go through all atoms */
	counter = 1;
        for ( atomindex=from; atomindex <= to; atomindex++)
          {
            /* get atom name&type */
            atomname = get_slice(atop->igraph, atomname, 4, 4, atomindex);
	    	    
            x = coords[atomindex*3];
	    y = coords[(atomindex*3)+1];
	    z = coords[(atomindex*3)+2];

            fprintf(pdb,"ATOM%7d  %4s%4s%5d    %8.3f%8.3f%8.3f\n", counter,atomname,resname,resindex+1,x,y,z);
	    
	    counter++;
          }
      }
    /* put "TER" at the very end of the file */
    fprintf(pdb, "TER\n");	  


/* close file and free memory */
close_file(pdb);

#ifdef USE_FREE
free(atomname);
free(resname);
#endif

#ifdef DEBUG_GRO
  printf("all O.K., %s closed\n", pdb_filename);
#endif
#ifdef DEBUG_MEM
  muntrace();
#endif
}

