// AMBGRO - program for convering AMBER topology to GROMACS topology 
// Filip Ryjacek 2002 

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <stddef.h>
#include <string.h>
#include <math.h>
/* #include <mcheck.h> */

#include "ambconv.h"


// write position constraints
void write_position_constraints(s_atop* atop)
{

  unsigned int counter;
  // atom parameters
  unsigned int atomindex;
  char* atomname;
  char* atomtype;

  // residue parameters 
  unsigned int from, to;
  unsigned int nres, resindex;
  char* reslabel;

  // force constants
  double fcx = 1000.0;
  double fcy = 1000.0;
  double fcz = 1000.0;

  atomname = (char*) xmalloc(5);
  atomtype = (char*) xmalloc(5);
  reslabel = (char*) xmalloc(5);

  fprintf(grotop, "\n [ position_restraints ]\n");

  reslabel = get_slice(atop->labres, reslabel, 4,4,0);

  nres = atop->nres;

  counter = 1;

  // go through all residues and print all atoms in each residue     
  for(resindex=0; resindex< nres; resindex++)
    {
      // get residue label and bounds 
      reslabel = get_slice(atop->labres, reslabel, 4, 4, resindex);

      from = atop->ipres[resindex]-1;
      if (resindex == atop->nres-1)
        {
          to = atop->natom-1;
        }
      else
        {
          to = (atop->ipres[resindex+1]-1)-1;
        }

  fprintf(grotop, "; %s\n",reslabel );
      
      // write atom info 
      for ( atomindex=from; atomindex <= to; atomindex++)
        {
          // get atom name&type 
          atomname = get_slice(atop->igraph, atomname,4, 4, atomindex);

          atomtype = get_slice(atop->isymbl, atomtype, 4, 4, atomindex);
	  
          fprintf(grotop, "%5d      1      %10.3f%10.3f%10.3f ; %5s%5s\n", counter,fcx,fcy,fcz,atomname,atomtype);
          counter++;
        }

    }


#ifdef USE_FREE
free(atomname);
free(atomtype);
free(reslabel);
#endif

#ifdef DEBUG_MEM
  muntrace();
#endif
}

